@@ -1,7 +1,13 @@
+import sys
 import os
+import re
 import itertools
+import shutil
+import subprocess
+import tempfile
 from pathlib import Path
 from typing import Any, List, Dict, Sequence, Optional
+from datetime import datetime
 from decimal import Decimal
 
 import click
@@ -152,12 +158,16 @@ def twistytimer(_json: bool, twistytimer_file: Path) -> None:
 
 
 config_dir = Path(os.environ.get("XDG_CONFIG_DIR", Path.home() / ".config"))
+scramble_history_config_dir = Path(
+    os.environ.get("SCRAMBLE_HISTORY_CONFIG_DIR", config_dir / "scramble_history")
+)
+if not scramble_history_config_dir.exists():
+    scramble_history_config_dir.mkdir(parents=True)
+
 
 # this needs to be a global path that user cant modify in click option
 # so _parse_merge_inputs can access it
-config_file = Path(
-    os.environ.get("SCRAMBLE_HISTORY_CONFIG", config_dir / "scramble_history.yaml")
-)
+config_file = scramble_history_config_dir / "files.yaml"
 
 
 def _parse_merge_inputs(
@@ -183,7 +193,9 @@ def _parse_merge_inputs(
 
 
 def _parse_query(
-    ctx: click.Context, param: click.Argument, value: Sequence[str],
+    ctx: click.Context,
+    param: click.Argument,
+    value: Sequence[str],
 ) -> Query:
     return parse_query(list(value))
 
@@ -192,18 +204,33 @@ def banner() -> None:
     click.echo("===================")
 
 
+KITTY_PATH = shutil.which("kitty")
+
+
+def _print_kitty_images(imgs: List[str]) -> bool:
+    if os.environ.get("TERM") != "xterm-kitty":
+        return False
+    printed = False
+    assert KITTY_PATH is not None, "could not find kitty on your path"
+    for img in imgs:
+        subprocess.run([KITTY_PATH, "icat", "--align=left", str(img)])
+        printed = True
+    return printed
+
+
 @main.command(
     context_settings=dict(
         ignore_unknown_options=True,
         allow_extra_args=True,
         max_content_width=110,
-    )
+    ),
+    short_help="merge solves together",
 )
 @click.option(
     "-s",
     "--sourcemap-file",
     help="Data file which saves choices on how to map solves from different sources",
-    default=config_dir / "scramble_history_sourcemap.json",
+    default=scramble_history_config_dir / "sourcemap.json",
     show_default=True,
     type=click.Path(dir_okay=False, path_type=Path),
 )
@@ -229,6 +256,19 @@ def banner() -> None:
     help="Group parsed results by key",
     default=None,
 )
+@click.option(
+    "-G", "--graph", is_flag=True, help="graph grouped results", default=False
+)
+@click.option(
+    "-O",
+    "--graph-opt",
+    default=(),
+    multiple=True,
+    type=click.Choice(
+        ["show", "save", "date-axis", "kitty-print", "annotate", "annotate-average"]
+    ),
+    help="graph options",
+)
 @click.option(
     "-q",
     "--query",
@@ -261,6 +301,8 @@ def banner() -> None:
 def merge(
     sourcemap_file: Path,
     action: str,
+    graph: bool,
+    graph_opt: Sequence[str],
     check: bool,
     sort_by: Optional[str],
     reverse: bool,
@@ -270,6 +312,12 @@ def merge(
 ) -> None:
     """
     merge solves from different data sources together
+
+    To provide input, either define a config file, or pass paths like:
+
+    \b
+    --cstimer ~/Downloads/cstimer.json
+    --twistytimer ~/Downloads/*twistytimer*.txt
     """
     from .source_merger import merge as merge_solves
 
@@ -294,8 +342,11 @@ def merge(
             # we just filtered, so set the solves to what the query returned
             solves = data
 
+    if graph and action != "stats":
+        action = "stats"
+
     res: Any = solves
-    if group_by is not None or action == "stats" and isinstance(res, list):
+    if (group_by is not None or action == "stats" or graph) and isinstance(res, list):
         if group_by is None:
             click.echo(
                 "Passed 'stats' with no '--group_by', grouping by 'event_description'",
@@ -303,6 +354,9 @@ def merge(
             )
             group_by = "event_description"
         key = str(group_by)
+        if len(solves) == 0:
+            click.echo("Solve list is empty!", err=True)
+            sys.exit(1)
         assert hasattr(
             solves[0], key
         ), f"Error: could not find attribute {key} on {solves[0]}"
@@ -320,11 +374,25 @@ def merge(
         header = f"Use {click.style('res', fg='green')} to review"
         IPython.embed(header=header)
     else:
-        from .group_operations import run_operations, grouped, find_best_group, operation_code, find_best
+        from .group_operations import (
+            run_operations,
+            grouped,
+            find_best_group,
+            operation_code,
+            find_best,
+            find_worst,
+        )
         from .timeformat import format_decimal
         from tabulate import tabulate
 
-        for group_name, group_solves in res.items():
+        # order by number of solves in the group
+        by_solve_count = sorted(
+            [(gn, len(gs)) for gn, gs in res.items()],
+            key=lambda k: k[1],
+            reverse=True,
+        )
+        for group_name, _ in by_solve_count:
+            group_solves = res[group_name]
             group_solves.sort(key=lambda s: s.when, reverse=reverse)
             banner()
             click.echo(group_name)
@@ -335,13 +403,18 @@ def merge(
                 if isinstance(recent_ao5, Exception)
                 else recent_ao5.describe_average()
             )
-            click.echo(f"Best => {find_best(group_solves).describe()}")
+            best_solve = find_best(group_solves).describe()
+            worst_solve = find_worst(group_solves).describe()
+            click.echo(f"Best => {best_solve}")
+            click.echo(f"Worst => {worst_solve}")
             click.echo(f"Most recent Ao5 => {desc}")
             click.echo(f"Solve Count => {len(group_solves)}")
             stat_data = run_operations(
                 group_solves, operation="average", counts=[5, 12, 50, 100]
             )
-            best = find_best_group(group_solves, operation="average", counts=[5, 12, 50, 100])
+            best = find_best_group(
+                group_solves, operation="average", counts=[5, 12, 50, 100]
+            )
             click.echo()
             click.echo(
                 tabulate(
@@ -358,6 +431,83 @@ def merge(
                     headers=(group_name, "Current", "Best"),
                 )
             )
+            if graph:
+                import seaborn as sns  # type: ignore[import]
+                import matplotlib.pyplot as plt  # type: ignore[import]
+                import pandas as pd  # type: ignore[import]
+
+                from dataclasses import asdict
+                from .models import State
+                from .error import unwrap
+
+                text = ""
+                if "annotate" in graph_opt:
+                    text += f"Best: {best_solve}\n"
+                    text += f"Worst: {worst_solve}\n"
+                    text += f"Count: {len(group_solves)}\n"
+
+                pd_input = pd.json_normalize(
+                    list(
+                        dict(list(asdict(solve).items()) + list({"solve": i}.items()))
+                        for i, solve in enumerate(group_solves)
+                        if solve.state == State.SOLVED
+                    )
+                )
+
+                filename = re.sub(
+                    r"[\?:\s\/\\]",
+                    r"_",
+                    f"{group_name}-{len(group_solves)}-{datetime.now().replace(microsecond=0)}.png",
+                )
+                _, ax = plt.subplots()
+                sns.lineplot(
+                    pd_input,
+                    x="when" if "date-axis" in graph_opt else "solve",
+                    y="full_time",
+                    ax=ax,
+                )
+                plt.xlabel("solve date" if "date-axis" in graph_opt else "solve #")
+                plt.ylabel("solve time")
+                textbox_props = dict(boxstyle="round", facecolor="lightblue", alpha=0.5)
+                if "annotate" in graph_opt:
+                    plt.annotate(
+                        text.strip(),
+                        xy=(0.05, 0.95),
+                        xycoords="axes fraction",
+                        fontsize=14,
+                        verticalalignment="top",
+                        bbox=textbox_props,
+                        color="black",
+                        horizontalalignment="left",
+                    )
+                if "annotate-average" in graph_opt:
+                    average = unwrap(grouped(solves, operation="average")).describe()
+                    plt.annotate(
+                        average,
+                        xy=(0.05, 0.05),
+                        xycoords="axes fraction",
+                        fontsize=10,
+                        verticalalignment="bottom",
+                        bbox=textbox_props,
+                        color="black",
+                        horizontalalignment="left",
+                    )
+                plt.title(group_name)
+                if "kitty-print" in graph_opt or "save" in graph_opt:
+                    # print using icat for the kitty terminal
+                    if "save" in graph_opt:
+                        click.echo(f"Saving to {filename}", err=True)
+                        plt.savefig(filename, format="png")
+                    else:
+                        with tempfile.TemporaryDirectory() as td:
+                            target = os.path.join(td, filename)
+                            plt.savefig(target, format="png")
+                            _print_kitty_images([target])
+                else:
+                    plt.show()
+                plt.clf()
+                plt.cla()
+                plt.close()
 
 
 if __name__ == "__main__":
